#ifndef GOOGLEX_GCAM_CREATIVE_CAMERA_PORTRAIT_MODE_PORTRAIT_CROS_WRAPPER_H_
#define GOOGLEX_GCAM_CREATIVE_CAMERA_PORTRAIT_MODE_PORTRAIT_CROS_WRAPPER_H_

// This header provides public API of the portrait mode feature for Chromium OS.
// Do not include any other headers if possible.

#include <cstdint>
#include <memory>

namespace creative_camera {

class PortraitProcessor;
class PortraitSegmenter;

class PortraitCrosWrapper {
 public:
  // Contains per-frame dynamic metadata defined in
  // https://developer.android.com/reference/android/hardware/camera2/CaptureResult
  struct FrameMetadata {
    // LENS_FOCAL_LENGTH
    float focal_length_mm = -1.0f;

    // LENS_FOCUS_DISTANCE
    float focus_distance_diopters = -1.0f;
  };

  // Contains camera static metadata defined in
  // https://developer.android.com/reference/android/hardware/camera2/CameraCharacteristics
  struct StaticMetadata {
    // LENS_FACING (0: FRONT, 1: BACK, 2: EXTERNAL, other: unknown)
    int lens_facing = -1;

    // SENSOR_INFO_PRE_CORRECTION_ACTIVE_ARRAY_SIZE
    int active_area_x0 = 0;
    int active_area_y0 = 0;
    int active_area_x1 = 0;
    int active_area_y1 = 0;

    // SENSOR_INFO_PHYSICAL_SIZE
    float sensor_physical_width_mm = -1.0f;
    float sensor_physical_height_mm = -1.0f;
  };

  // Defines metadata for a portrait request.
  //
  // `width` and `height` specifies the dimensions of input and output images.
  // `orientation` can be 0, 90, 180, or 270 in degrees counter-clockwise.
  //
  // `enable_rectiface` turns on/off face distortion correction. When it's on,
  // below metadata is needed:
  // `frame_metadata` specifies metadata of this captured frame;
  // `static_metadata` specifies metadata of the capturing device.
  struct Request {
    int width = 0;
    int height = 0;
    int orientation = 0;
    bool enable_rectiface = false;
    FrameMetadata frame_metadata;
    StaticMetadata static_metadata;
  };

  PortraitCrosWrapper();
  ~PortraitCrosWrapper();

  // Initializes the portrait processor. Should be called before Process().
  // Returns true on success.
  bool Init();

  // Processes a portrait image, returning a background defocused result.
  //
  // `id` should be a unique identifier for this portrait. It should not be
  // duplicated within the lifetime of this PortraitProcessor.
  //
  // `rgb_input_data` and `rgb_output_data` point to memory of size
  // `request.width`x`request.height`x3 for storing input and output images in
  // RGB planar format. The memory must remain valid until Process() returns.
  //
  // Returns true on success.
  bool Process(int64_t id, const Request& request,
               const uint8_t* rgb_input_data, uint8_t* rgb_output_data);

 private:
  std::unique_ptr<PortraitProcessor> processor_;
  std::unique_ptr<PortraitSegmenter> segmenter_;
};

}  // namespace creative_camera

#endif  // GOOGLEX_GCAM_CREATIVE_CAMERA_PORTRAIT_MODE_PORTRAIT_CROS_WRAPPER_H_
